<?php //$Id: restorelib.php,v 1.20 2012-01-18 16:53:37 juanca Exp $

require_once dirname(__FILE__).'/locallib.php';
require_once dirname(__FILE__).'/fields_list.php';

//This code is based on assignment module
function vpl_restore_mods($mod,$restore) {

	global $CFG;
	
	debugging("Restoring vpl id => {$mod->id}", DEBUG_DEVELOPER);
	
	$status = true;
	
	//Get record from backup_ids
	$data = backup_getid($restore->backup_unique_code,$mod->modtype,$mod->id);

	if ($data && $data->new_id == 0) {
		//Now get completed xmlized object
		$info = $data->info;
		$pinfo = $info['MOD']['#'];
		//if necessary, write to restorelog and adjust date/time fields
		if (isset($restore->course_startdateoffset)) {
			restore_log_date_changes('vpl', $restore, $pinfo, vpl_dates_fields());
		}

		//Now, we build the vpl record structure

		$vpl = new stdClass();
		$vpl->course = $restore->course_id;
		$fields = vpl_fields();
		foreach($fields as $field){
			$tag = strtoupper($field);
			$vpl->$field = backup_todb($pinfo[$tag]['0']['#']);
		}

		//We have to recode the grade field if it is <0 (scale)
		if ($vpl->grade < 0) {
			$scale = backup_getid($restore->backup_unique_code,"scale",abs($vpl->grade));
			if ($scale) {
				$vpl->grade = -($scale->new_id);
			}
		}

		$newid = insert_record ("vpl",$vpl);
		if ($newid) {
			//update backup_ids with new one
			backup_putid($restore->backup_unique_code,$mod->modtype,$mod->id, $newid);
			//restore variations
			$status = vpl_restore_variations($mod->id, $newid,$info,$restore) && $status;
			//Restore config files
			vpl_restore_config_files ($mod->id, $newid, $restore);
			//Do some output
			if (!defined('RESTORE_SILENTLY')) {
				echo "<li>".get_string("modulename","vpl")." \"".format_string(stripslashes($vpl->name),true)."\"</li>";
				flush();
			}
			if (restore_userdata_selected($restore,'vpl',$mod->id)) {
				//Restore vpl_assignned_variations
				$status = vpl_restore_assigned_variations($mod->id, $newid,$info,$restore) && $status;
				//Restore vpl_submissions
				$status = vpl_restore_submissions($mod->id, $newid,$info,$restore) && $status;
			}
			//We have to recode the basedon field if it is >0
			//To avoid recursive basedon we update basedon after insertion
			//If needed we restore basedon module
			if ($vpl->basedon > 0) {
				$basedon = backup_getid($restore->backup_unique_code,"vpl",$vpl->basedon);
				if ($basedon) {
					if($basedon->new_id==0){
						//Load vpl basedon instance needed
						$modneeded = new stdClass();
						$modneeded->modtype = 'vpl';
						$modneeded->id = $vpl->basedon;
						//Recursive call
						vpl_restore_mods($modneeded,$restore);
						//Info basedon reload
						$basedon = backup_getid($restore->backup_unique_code,"vpl",$vpl->basedon);
					}
					if($basedon->new_id>0){
						//Recoded basedon module
						$vpl->basedon = $basedon->new_id;
						$vpl->id=$newid;
						//Update vpl record
						update_record('vpl',$vpl);
					}else{
						print_box("Error loading basedon needed module");
					}
				}
			}
		} else {
			$status = false;
		}
	}

	debugging("Restoring vpl result {$status}", DEBUG_DEVELOPER);
	if($status == false){
		print_box('Error restoring VPL instance '.s($vpl->name));
	}
	return true;
}

//This function restores the vpl_variatons
function vpl_restore_variations($old_vpl_id, $new_vpl_id,$info,$restore) {
	global $CFG;
	
	debugging("Restoring variations of vpl id => {$old_vpl_id}", DEBUG_DEVELOPER);

	$status = true;

	//Get the variations array - it might not be present
	if (isset($info['MOD']['#'][VPL_TAG_VARIATIONS]['0']['#'][VPL_TAG_VARIATION])) {
		$variations = $info['MOD']['#'][VPL_TAG_VARIATIONS]['0']['#'][VPL_TAG_VARIATION];
	} else {
		return true;
	}

	$fields = vpl_variations_fields();
	//Iterate over submissions
	$i=0;
	foreach($variations as $sub_info) {
		$i++;
		$oldid = backup_todb($sub_info['#']['ID']['0']['#']);
		
		debugging("Restoring vpl variations id => {$oldid}", DEBUG_DEVELOPER);
		
		//Now, build the vpl_variations record structure
		$variation = new stdClass();
		foreach($fields as $field){
			$tag = strtoupper($field);
			$variation->$field = backup_todb($sub_info['#'][$tag]['0']['#']);
		}
		//Recode vpl field with the new value
		$variation->vpl = $new_vpl_id;
		
		//The structure is equal to the db, so insert the vpl_submission
		$newid = insert_record (VPL_VARIATIONS,$variation);

		if ($newid) {
			//We have the newid, update backup_ids
			backup_putid($restore->backup_unique_code,VPL_VARIATIONS,$oldid,$newid);
		} else {
			$status = false;
		}
	}
	debugging("Restoring vpl variations result {$status}", DEBUG_DEVELOPER);
	return $status;
}


//This function restores variations assigned to users
function vpl_restore_assigned_variations($old_vpl_id, $new_vpl_id,$info,$restore) {
	global $CFG;
	
	//Get the submissions array - it might not be present
	if (isset($info['MOD']['#'][VPL_TAG_ASSIGNED_VARIATIONS]['0']['#'][VPL_TAG_ASSIGNED_VARIATION])) {
		$assigned_variations = $info['MOD']['#'][VPL_TAG_ASSIGNED_VARIATIONS]['0']['#'][VPL_TAG_ASSIGNED_VARIATION];
	} else {
		return true;
	}
	
	debugging("Restoring assigned variations of vpl id => {$old_vpl_id}", DEBUG_DEVELOPER);
	
	$status = true;
	$fields = vpl_assigned_variations_fields();
	//Iterate over assigned variations
	$i=0;
	foreach($assigned_variations as $sub_info) {
		$i++;
		//Now, build the VPL_ASSIGNED_VARIATIONS record structure
		$assigned_variation = new stdClass();
		foreach($fields as $field){
			$tag = strtoupper($field);
			$assigned_variation->$field = backup_todb($sub_info['#'][$tag]['0']['#']);
		}
		debugging("Restoring assigned variation => ".print_r($assigned_variation,true), DEBUG_DEVELOPER);
		
		//Recode vpl field with the new value
		$assigned_variation->vpl = $new_vpl_id;
		
		//We have to recode the variation field
		$variation = backup_getid($restore->backup_unique_code,VPL_VARIATIONS,$assigned_variation->variation);
		if ($variation) {
			$assigned_variation->variation = $variation->new_id;
		}
		//We have to recode the userid field
		$user = backup_getid($restore->backup_unique_code,"user",$assigned_variation->userid);
		if ($user) {
			$assigned_variation->userid = $user->new_id;
		}

		//The structure is equal to the db, so insert the vpl_assigned_variation
		$newid = insert_record (VPL_ASSIGNED_VARIATIONS,$assigned_variation);

		//Do some output
		if ($i % 5 == 0) {
			if (!defined('RESTORE_SILENTLY')) {
				echo ".";
				if ($i % 100 == 0) {
					echo "<br />";
				}
				flush();
			}
		}

		if (!$newid) {
			$status = false;
		}
	}

	debugging("Restoring assigned variations result {$status}", DEBUG_DEVELOPER);
	return $status;
}


//This function restores the vpl_submissions
function vpl_restore_submissions($old_vpl_id, $new_vpl_id,$info,$restore) {
	global $CFG;
	
	debugging("Restoring submissions of vpl id => {$old_vpl_id}", DEBUG_DEVELOPER);
	
	$status = true;

	//Get the submissions array - it might not be present
	if (isset($info['MOD']['#'][VPL_TAG_SUBMISSIONS]['0']['#'][VPL_TAG_SUBMISSION])) {
		$submissions = $info['MOD']['#'][VPL_TAG_SUBMISSIONS]['0']['#'][VPL_TAG_SUBMISSION];
	} else {
		$submissions = array();
	}

	$fields = vpl_submission_fields();
	//Iterate over submissions
	$i=0;
	foreach($submissions as $sub_info) {
		$i++;
		$oldid = backup_todb($sub_info['#']['ID']['0']['#']);
		$olduserid = backup_todb($sub_info['#']['USERID']['0']['#']);

		//Now, build the vpl_SUBMISSIONS record structure
		$submission = new stdClass();
		foreach($fields as $field){
			$tag = strtoupper($field);
			$submission->$field = backup_todb($sub_info['#'][$tag]['0']['#']);
		}
		debugging("Restoring submission => ".print_r($submission,true), DEBUG_DEVELOPER);

		//Recode vpl field with the new value
		$submission->vpl = $new_vpl_id;
		
		//We have to recode the userid field
		$user = backup_getid($restore->backup_unique_code,"user",$submission->userid);
		if ($user) {
			$submission->userid = $user->new_id;
		}
		if($submission->grader != 0){ //Not null or automatic grading
			//We have to recode the grader field
			$user = backup_getid($restore->backup_unique_code,"user",$submission->grader);
			if ($user) {
				$submission->grader = $user->new_id;
			}
		}

		//The structure is equal to the db, so insert the vpl_submission
		$newid = insert_record ("vpl_submissions",$submission);

		//Do some output
		if ($i % 5 == 0) {
			if (!defined('RESTORE_SILENTLY')) {
				echo ".";
				if ($i % 100 == 0) {
					echo "<br />";
				}
				flush();
			}
		}

		if ($newid) {
			//We have the newid, update backup_ids
			backup_putid($restore->backup_unique_code,"vpl_submissions",$oldid,$newid);

			//Now copy moddata associated files
			$status = vpl_restore_submission_files ($old_vpl_id, $new_vpl_id,
						$olduserid, $submission->userid,
						$oldid,$newid, $restore);

		} else {
			$status = false;
		}
	}
	debugging("Restoring submissions result {$status}", DEBUG_DEVELOPER);
	
	return $status;
}

/**
 * Restore de config dir of a vpl instance
 * @param $oldvplid id at backup
 * @param $newvplid id restored
 * @param $restore info object
 * @return bool (true==OK, false == error)
 */
function vpl_restore_config_files ($oldvplid, $newvplid, $restore) {
	global $CFG;

	$status = true;
	$from_dir = $CFG->dataroot.'/temp/backup/'.$restore->backup_unique_code.
                         '/moddata/vpl/'.$oldvplid.'/config';
	//Check it exists
	if (is_dir($from_dir)) {
		$to_dir = $CFG->dataroot.'/vpl_data/'.$restore->course_id.'/'.$newvplid.'/config';
		$status = check_dir_exists($to_dir,true,true);
		$status = backup_copy_file($from_dir, $to_dir) && $status;
	}
	debugging("Restoring config files {$status}", DEBUG_DEVELOPER);
	return $status;
}

//This function copies the vpl related info from backup temp dir to course moddata folder,
//creating it if needed and recoding everything (vpl id and user id)
/**
 * Restore submission files
 * @param $old_vpl_id vpl instance id at backup
 * @param $new_vpl_id restored vpl instance id
 * @param $old_user_id user id at backup
 * @param $new_user_id restored user id
 * @param $old_id submission instance id at backup
 * @param $new_id restored submission id
 * @param $restore info object
 * @return bool (true==OK, false == error)
 */
function vpl_restore_submission_files ($old_vpl_id, $new_vpl_id,
$old_user_id, $new_user_id,
$old_id, $new_id, $restore) {

	global $CFG;

	$status = true;
	$from_dir = $CFG->dataroot.'/temp/backup/'.$restore->backup_unique_code.
                         '/moddata/vpl/'.$old_vpl_id.'/usersdata/'.$old_user_id.'/'.$old_id;
	//Check it exists
	if (is_dir($from_dir)) {
		$to_dir = $CFG->dataroot.'/vpl_data/'.$restore->course_id.'/'.
		$new_vpl_id.'/usersdata/'.$new_user_id.'/'.$new_id;
		$status = check_dir_exists($to_dir,true,true);
		$status = backup_copy_file($from_dir, $to_dir) && $status;
	}
	return $status;
}

//Return a content decoded to support interactivities linking. Every module
//should have its own. They are called automatically from
//vpl_decode_content_links_caller() function in each module
//in the restore process
function vpl_decode_content_links ($content,$restore) {

	global $CFG;

	$result = $content;

	//Link to the list of vpls

	$searchstring='/\$@(VPLINDEX)\*([0-9]+)@\$/';
	//We look for it
	preg_match_all($searchstring,$content,$foundset);
	//If found, then we are going to look for its new id (in backup tables)
	if ($foundset[0]) {
		//Iterate over foundset[2]. They are the old_ids
		foreach($foundset[2] as $old_id) {
			//We get the needed variables here (course id)
			$rec = backup_getid($restore->backup_unique_code,"course",$old_id);
			//Personalize the searchstring
			$searchstring='/\$@(VPLINDEX)\*('.$old_id.')@\$/';
			//If it is a link to this course, update the link to its new location
			if($rec->new_id) {
				//Now replace it
				$result= preg_replace($searchstring,$CFG->wwwroot.'/mod/vpl/index.php?id='.$rec->new_id,$result);
			} else {
				//It's a foreign link so leave it as original
				$result= preg_replace($searchstring,$restore->original_wwwroot.'/mod/vpl/index.php?id='.$old_id,$result);
			}
		}
	}

	//Link to vpl view by moduleid

	$searchstring='/\$@(VPLVIEWBYID)\*([0-9]+)@\$/';
	//We look for it
	preg_match_all($searchstring,$result,$foundset);
	//If found, then we are going to look for its new id (in backup tables)
	if ($foundset[0]) {
		//Iterate over foundset[2]. They are the old_ids
		foreach($foundset[2] as $old_id) {
			//We get the needed variables here (course_modules id)
			$rec = backup_getid($restore->backup_unique_code,"course_modules",$old_id);
			//Personalize the searchstring
			$searchstring='/\$@(VPLVIEWBYID)\*('.$old_id.')@\$/';
			//If it is a link to this course, update the link to its new location
			if($rec->new_id) {
				//Now replace it
				$result= preg_replace($searchstring,$CFG->wwwroot.'/mod/vpl/view.php?id='.$rec->new_id,$result);
			} else {
				//It's a foreign link so leave it as original
				$result= preg_replace($searchstring,$restore->original_wwwroot.'/mod/vpl/view.php?id='.$old_id,$result);
			}
		}
	}

	return $result;
}

//This function makes all the necessary calls to xxxx_decode_content_links()
//function in each module, passing them the desired contents to be decoded
//from backup format to destination site/course in order to mantain inter-activities
//working in the backup/restore process. It's called from restore_decode_content_links()
//function in restore process
function vpl_decode_content_links_caller($restore) {
	global $CFG;
	$status = true;
	$vpls = get_records_sql ("SELECT id, shortdescription
                                   FROM {$CFG->prefix}vpl
                                   WHERE course = {$restore->course_id}");
	if ($vpls) {
		//Iterate over each vpl->shortdescription
		$i = 0;   //Counter to send some output to the browser to avoid timeouts
		foreach ($vpls as $vpl) {
			//Increment counter
			$i++;
			$content = $vpl->shortdescription;
			$result = restore_decode_content_links_worker($content,$restore);
			if ($result != $content) {
				//Update record
				$vpl->shortdescription = addslashes($result);
				$status = update_record("vpl",$vpl);
				if (debugging()) {
					if (!defined('RESTORE_SILENTLY')) {
						echo '<br /><hr />'.s($content).'<br />changed to<br />'.s($result).'<hr /><br />';
					}
				}
			}
			//Decode fulldescription.html file
			$fulldescription = $CFG->dataroot.'/vpl_data/'.$restore->course_id.'/'.$vpl->id;
			$fulldescription .='/config/fulldescription.html';
			if(file_exists($fulldescription)){
				$data = file_get_contents($fulldescription);
				$data = restore_decode_content_links_worker($data,$restore);
				file_put_contents($fulldescription,$data);
			}

			//Do some output
			if ($i % 5 == 0) {
				if (!defined('RESTORE_SILENTLY')) {
					echo ".";
					if ($i % 100 == 0) {
						echo "<br />";
					}
					if (defined('RESTORE_SILENTLY_NOFLUSH')) {
						flush();
        			}
				}
			}
		}
	}
	return $status;
}

/**
 * This function returns a log record with all the necessay transformations
 * done. It's used by restore_log_module() to restore modules log
 * @param $restore object info
 * @param $log log record to recode to restore
 * @return mix false or log record updated
 */
function vpl_restore_logs($restore,$log) {
	$status = false;
	//Decode url field
	if($log->action == 'view all'){
		$log->url='index.php?id='.$log->course;
		$status = $log;
	}else{
		if(strpos($log->url,'userid=')){
			if(ereg('(.*)\\?id\\=([0-9]*).*userid\\=([0-9]*)',$log->url,$res)){
				$mod = backup_getid($restore->backup_unique_code,'user',$res[3]);
				if($mod){ //User restored
					$log->url=$res[1].'?id='.$log->cmid.'&amp;userid='.$mod->new_id;
				}else{ //User in local moodle
					$log->url=$res[1].'?id='.$log->cmid.'&amp;userid='.$res[3];
				}
				$status = $log;
			}
		}else{
			if(ereg('(.*)\\?id\\=([0-9]*)',$log->url,$res)){
				$log->url=$res[1].'?id='.$log->cmid;
				$status = $log;
			}
		}
	}
	return $status;
}
?>